from cv2 import cv
import numpy 
import sqlite3
from scipy.ndimage import *
import tesseract
import sys
from Queue import PriorityQueue
from threading import Thread
import thread
from pylab import *
import os
from copy import copy
from random import randint
from math import ceil
from analyze_caption import FindMatch
import time
import json
from utils import *
from frame_ocr import FrameOcr


##break file into frames and feed files into ocr
##Params
##videofile - string containing path of the video to be analyzed
##startTime - string indicating the time of day the video was started formatted. This time should be formatted "Hour,Minute"
##FPS - integer indicating at what frame rate the video was recorded
def BreakIntoFrames(videofile,startTime, fps):
	
	sceneFrames = [] #frames from current scene
	changeVector = numpy.array([0]) #numerical changes between successive frames 
	cap = cv.CaptureFromFile(videofile)
	success = 1
	prevFrame = 0
	frameNum = 1
	
	while success:
		success = cv.GrabFrame(cap) #grab a frame at a time
		if not success:
			break
		
		currFrame = cv.RetrieveFrame(cap)

		#we will represent a frameInfo as {the image, the frame number, the time the frame was recorded}
		frameInfo = {'img' : currFrame, 'frameNum': frameNum, 'frameTime': GetFrameTime(frameNum,startTime,fps)}
		sceneFrames.append(frameInfo)	
	
		#convert from IIpimage to matrix of pixels so that the difference between frames can be calculated
		mat = cv.CreateMat(currFrame.height, currFrame.width, cv.CV_32FC3)
		cv.Convert(currFrame, mat)
		currFrameArr = numpy.array(mat)

		###################Scene Detection#########################################
		if frameNum > 1:
			change = CompareImages(prevFrame,currFrameArr) 
			changeVector = numpy.append(changeVector, change)
			
			changeMean = mean(changeVector[-200:]) #calculate the mean of the last 200 changes
			changeStdDev = standard_deviation(changeVector[-200:]) 
			upperLimit = changeMean + 3*changeStdDev
			lowerLimit = changeMean - 3*changeStdDev
			
			#when a scene is detected
			if change > upperLimit:
				#send one percent of the frames from current scene to ocr
				ocrThread = Thread(target = ocr, args = [GetRandomSubset(sceneFrames,.01)])
				ocrThread.setDaemon(True)
				ocrThread.start()
				ocrThread.join()
				sceneFrames = []
		###########################End Scene Detection##########################################
		frameNum += 1
		prevFrame = currFrameArr	
	return sceneFrames


global ocrTimeList
ocrTimeList  = []   ##keep track of times for evaluating the speed
ocrLock = thread.allocate_lock()

##ocr
##Params:
## frames - a list containing frame info dicts {img,framenum,frametime}
def ocr(frames):
		
	for img in frames:
		frameInfo = copy(img)
		frameNum = img['frameNum']
		img = img['img']
		cv.SaveImage( SENS_FRAME_DIR + str(frameNum) +'.png',frameInfo['img']) ##save the image as png

		ocrStart = time.time()		
		text = FrameOcr(frameNum)  ##get the ocr
		#####keep track of times ###########################
		ocrTime = time.time() - ocrStart
		ocrLock.acquire()
		ocrTimeList.append(ocrTime)
		ocrLock.release()
		#determine if frame is sensitive. Replace this with better module
		if FindMatch(text) >= .05:
			WriteToFile(frameInfo, text)
		else:
			os.remove(SENS_FRAME_DIR + str(frameNum) +'.png')




#Lock to make sure two threads don't write to Json file at the same time
global jsonLock
jsonLock = thread.allocate_lock()
##WriteToFile
##Params
##frameInfo -  frame info dicts {img,frame number, frame time}
##frameText - text extracted from frame
##This function will generate the json file required by the UI
def WriteToFile(frameInfo, frameText):
	global videoId  ##get the Id of the video that was analyzed (for database purposes)
	conn = sqlite3.Connection('Process')
	curs = conn.cursor() 
	
	
	##grab the processes that were running when this frame was recorded
	frameTime = frameInfo['frameTime']
	frameNum = frameInfo['frameNum']
	processes = ''
	curs.execute('SELECT running FROM process WHERE time = ? AND id = ?', [frameTime, videoId])
	qResult = curs.fetchall()
	curs.close()	
	conn.close()

	resDict = dict()
	maxLength = 0
	for result in qResult:
		resLength = len(result[0])
		if resLength > maxLength:
			processes = result[0] +'\n'
			maxLength = resLength

	##date should be changed somehow
	lclTime = time.localtime()
	timeField = str(lclTime.tm_year) + ',' +str(lclTime.tm_mon) + ',' +str(lclTime.tm_wday) + ',' + frameTime
	
	resDict['startDate'] = timeField
	resDict['headline'] = str(frameNum)
	resDict['text']  = frameText
	resDict['process'] = processes
	resDict['asset'] = {'media': str(frameNum) + '.png'}
	
	jsonLock.acquire()
	f = open(JSON_FILE, 'a')
	f.write(json.dumps(resDict) + ',\n')
	f.close()
	jsonLock.release()
	


def GetFrameTime(frameNum, startTime, fps):

	startHour = int(startTime.split(',')[0])
	startMinute = int(startTime.split(',')[1])
	
	frameMin = (startMinute+(frameNum/fps))%60
	carry = (startMinute + (frameNum/fps) )/60
	frameTime = str(startHour+carry) + ',' + str(frameMin)
	return frameTime


if __name__ == '__main__':
	if len(sys.argv)  != 4:
		print 'caption.py [videofile] [video id] [frame rate]'
		exit()
	else:
		global videoId
		
		fps = sys.argv[3]
		videoId = sys.argv[2]
		filename = sys.argv[1]

		conn = sqlite3.Connection('Process')
		c = conn.cursor()
		c.execute('SELECT time from process where id =?', [int(sys.argv[2])])
		startTime = str(c.fetchone()[0])
		c.close()
		conn.close()	

		
		mkdir_p(SENS_FRAME_DIR)	
		
		header = '{\n"timeline": {"headline":  "Insider Threat in VNC", "type": "default", "text": "Intro", "date": [   \n'
		footer =  ']}}'
		f = open(JSON_FILE, 'w')
		f.write(header+ '\n')
		f.close()
		progStartTime = time.time() 		
		BreakIntoFrames(filename,startTime,int(fps))

		##################Stats###############################################
		programTime = time.time() - progStartTime #grab the overall time to analyze the video
		slowOcr = max(ocrTimeList)		
		fastOcr = min(ocrTimeList)
		avgOcr = sum(ocrTimeList)/len(ocrTimeList)
		resultLog = str()
		resultLog +=  "\nProgram Time: " + str(programTime)
		resultLog += "\nFastest OCR: " + str(fastOcr)
		resultLog += "\nSlowest OCR: " + str(slowOcr)
		resultLog += "\nAverage OCR: " + str(avgOcr)	
		f = open('Result Log', 'w')
		f.write(resultLog)	
		f.close()
		print resultLog

		f = open(JSON_FILE, 'r')
		fileText =  f.read()
		fileText = fileText[:-2]
		f.close()
		
		f = open(JSON_FILE,'w')
		f.write(fileText +footer + '\n')
		f.close()

