from cv2 import cv
import numpy 
from scipy.ndimage import *
import tesseract
import sys
from Queue import PriorityQueue
from threading import Thread
import thread
from pylab import *
import os
from copy import copy
from random import randint
from math import ceil
import os, errno

DICT_NAME = 'newwords2'
unigrams = dict()
bigrams = dict()
trigrams = dict()

def LoadDicts():
	wordFile = open(DICT_NAME, 'r')
	for line in wordFile:
		line = line.lstrip('\n').rstrip('\n')
		words = line.split(' ')
		if len(words) == 1:
			unigram[line] = 1
		else if len(words) == 2:
			bigrams[line] = 1
		else if len(words) == 3:
			trigrams[line] = 1 


##output a metric describing how likely that the subject of the 
##given string is the same as the dictionary
def FindMatch(text):	
	uniCount = biCount = triCount= 0	
	totalWords=0
	for line in text.split('\n'):
			words = line.split(' ')
			wnum = len(words)
			totalWords += wnum
			#traverse line and determine how many unigrams, bigrams, and trigrams exist
			for i in range(wnum):
				if words[i] in unigrams:
					uniCount += 1
				if i+1<wnum and words[i] + ' ' + words[i+1]:
					biCount += 1				
				if i+2<wnum and words [i] + ' ' + words[i+1] + ' ' + words[i+2]:
					triCount += 1
	uniPerc = uniCount/ totalWords
	biPerc = biCount/ (totalWords-1)
	triPerc = triCount/(totalWords-2)

	return uniPerc+biPerc+triPerc

 
				


def GetRandomSubset(alist,prop):
	length = len(alist)
	subset = []
	for i in range(int(ceil(length*prop))):
		randIndex = randint(0,length-1)
		while alist[randIndex]  in  subset:
			randIndex = randint(0, length-1)
		subset.append(alist[randIndex])
	return subset


done = 0
frames = []
#break file into frames while placing them in global queue
def BreakIntoFrames(videofile):
	sceneFrames = []
	changeVector = numpy.array([0])
	cap = cv.CaptureFromFile(videofile)
	success = 1
	prevFrame = 0
	frameNum = 1
	while success:
		success = cv.GrabFrame(cap)
		if not success:
			break
		
		currFrame = cv.RetrieveFrame(cap)
		sceneFrames.append((currFrame,frameNum))	
		#print frameNum
		
		mat = cv.CreateMat(currFrame.height, currFrame.width, cv.CV_32FC3)
		cv.Convert(currFrame, mat)
		currFrameArr = numpy.array(mat)
		
		if frameNum > 1:
			change = CompareImages(prevFrame,currFrameArr)
			changeVector = numpy.append(changeVector, change)
			
			changeMean = mean(changeVector[-200:]) #calculate the mean of the last 200 changes
			changeStdDev = standard_deviation(changeVector[-200:]) 
			upperLimit = changeMean + 3*changeStdDev
			lowerLimit = changeMean - 3*changeStdDev

			if change > upperLimit:
				ocrThread = Thread(target = ocr, args = [GetRandomSubset(sceneFrames,.1)])
				ocrThread.setDaemon(True)
				ocrThread.start()
				ocrThread.join()
				sceneFrames = []
				
		frameNum += 1
		prevFrame = currFrameArr	
	return sceneFrames

def rgb2gray(image):
	return ((image[:,:,0]+image[:,:,1]+image[:,:,2])/3)

def CompareImages(image1, image2):
	maxchangeframe = 1
	diff = abs(rgb2gray(image1) - rgb2gray(image2))
	if diff.sum() != 0:
		change = diff.sum()/(diff.size * maxchangeframe)
	else:
		change = 0

	return change

def ocr(frames):

	api = tesseract.TessBaseAPI()
	api.Init(".","eng",tesseract.OEM_DEFAULT)
	api.SetPageSegMode(tesseract.PSM_AUTO)
	text = ""	

	for img in frames:
		frameNum = img[1]
		img = img[0]
		threshold=140
		colour=255
		cv.Threshold(img,img, threshold,colour,cv.CV_THRESH_BINARY) 
		img_processed = cv.CreateImage((img.width * 2,img.height *2), img.depth, img.nChannels)
		cv.Resize(img, img_processed)


		print "ocring"
		tesseract.SetCvImage(img_processed,api)
		text = api.GetUTF8Text()
		cv.SaveImage( './screencap/' + str(frameNum) +'.png',img_processed)
		text += '\n' + 'Metric = ' + str(FindMatch(text))

		f = open( './screencap/' + str(frameNum) +'.kd', 'w')
		f.write(text)
		f.close()
		conf=api.MeanTextConf()	
	
	


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc: # Python >2.5
        if exc.errno == errno.EEXIST:
            pass
        else: raise


if __name__ == '__main__':
	if len(sys.argv) != 2:
		print 'Provide Video File'
	else:
		filename = sys.argv[1]
		mkdir_p('./screencap/')				
		BreakIntoFrames(filename)

		

